from pathlib import Path
from typing import NamedTuple, Callable
import logging

from .tasks import Task

logger = logging.getLogger(__name__)


class ConversionConfig(NamedTuple):
    input_dir: Path
    output_dir: Path
    output_ext: str
    input_ext: str


def _get_converted_path(input_path: Path, config: ConversionConfig):
    """
    Return the path you would obtain if you moved the file in the input
    path to the output directory in the config and changed its extension.
    """

    relative_path = input_path.relative_to(config.input_dir)
    output_filename = str(input_path.stem) + f".{config.output_ext}"
    output_filename.replace("'", "")
    output_path = config.output_dir / relative_path.parent / output_filename
    return output_path


def _build_conversion_tasks(
    input_files: list[Path],
    output_files: list[Path],
    config: ConversionConfig,
    conversion_func: Callable,
) -> list[Tasks]:
    tasks = []
    for input_path, output_path in zip(input_files, output_files):
        cmd = conversion_func(input_path, config.output_dir, config.output_ext)
        tasks.append(Task(cmd, output_tmp, output_path))
    return tasks


def get_unconverted_files(
    input_files: list[Path], config: ConversionConfig
) -> list[Path]:
    output_files = [_get_converted_path(f, config) for f in input_files]
    return [f for f in output_files if not f.exists()]


def convert(config: ConversionConfig, conversion_func: Callable):

    logger.info("Converting files in %s", config.input_dir)
    logger.info("Writing output to: %s", config.output_dir)

    input_files = get_files_recursive(config.input_dir.resolve(), config.input_ext)
    output_files = get_uncoverted_files(input_files, config)

    tasks = _build_conversion_tasks(input_files, output_files, config, conversion_func)
    run_tasks(tasks)
